#include "lemire.h"


LemireMaxClampedFilter* lemire_max_clamped_init(int windowSize, int dataLength) {
    LemireMaxClampedFilter* lem = NULL;
    if (windowSize % 2 == 0) {
        return NULL;
    }
    lem = malloc(sizeof(LemireMaxClampedFilter));
    if (lem == NULL) {
        return NULL;
    }
    lem->windowSize = windowSize;
    lem->dataToFilterSize = (dataLength + windowSize - 1);
    lem->fifo = deq_int_init();
    lem->dataToFilter = malloc(lem->dataToFilterSize * sizeof(float));
    memset(lem->dataToFilter, 0, lem->dataToFilterSize * sizeof(float));
    return lem;
}

void lemire_max_clamped_free(LemireMaxClampedFilter* l) {
    if (l != NULL) {
        free(l->dataToFilter);
        l->dataToFilter = NULL;
        deq_int_free(&l->fifo);
        free(l);
    }
}

void lemire_max_clamped_filter(LemireMaxClampedFilter* l, float* restrict array, size_t array_size, float* restrict max) {
    int ix = l->windowSize / 2;
    //TODO: avoid copy entirely?
    memcpy(&l->dataToFilter[ix], &array[0], array_size * sizeof(float));
    for (int j = 0; j < ix; ++j){
        l->dataToFilter[j] = array[0];
    }
    for (int j = l->dataToFilterSize - ix; j < l->dataToFilterSize; ++j){
        l->dataToFilter[j] = array[array_size - 1];
    }

    deq_int_clear(&l->fifo);
    deq_int_push_back(&l->fifo, 0);

    for (int i = 1; i < l->windowSize; ++i) {
        if (l->dataToFilter[i] > l->dataToFilter[i - 1]) { //overshoot
            deq_int_pop_back(&l->fifo);
            while (!deq_int_empty(&l->fifo)){
                if (l->dataToFilter[i] <= l->dataToFilter[*deq_int_back(&l->fifo)]) {
                    break;
                }
                deq_int_pop_back(&l->fifo);
            }
        }

        deq_int_push_back(&l->fifo, i);
    }

    for (int i = l->windowSize; i < l->dataToFilterSize; ++i) {
        max[i - l->windowSize] = l->dataToFilter[*deq_int_front(&l->fifo)];

        if (l->dataToFilter[i] > l->dataToFilter[i - 1]) { //overshoot
            deq_int_pop_back(&l->fifo);
            while (!deq_int_empty(&l->fifo)){
                if (l->dataToFilter[i] <= l->dataToFilter[*deq_int_back(&l->fifo)]) {
                    break;
                }
                deq_int_pop_back(&l->fifo);
            }
        }

        deq_int_push_back(&l->fifo, i);

        if (i == (l->windowSize + *deq_int_front(&l->fifo))){
            deq_int_pop_front(&l->fifo);
        }
    }

    max[l->dataToFilterSize - l->windowSize] = l->dataToFilter[*deq_int_front(&l->fifo)];
}